# SPDX-FileCopyrightText: © 2024 The "Whiteprints template context" contributors <whiteprints@pm.me>
#
# SPDX-License-Identifier: MIT

"""Hatch build hook for localization."""

import sys
from pathlib import Path

from babel.messages.frontend import OptionError
from babel.messages.setuptools_frontend import compile_catalog
from hatchling.builders.config import BuilderConfigBound
from hatchling.builders.hooks.plugin.interface import BuildHookInterface


if sys.version_info < (3, 12):
    from typing_extensions import override
else:
    from typing import override


def _compile(locale_path: Path) -> None:
    """Compile a localization file."""
    cmd = compile_catalog()
    cmd.initialize_options()
    # Hatchling's typing sucks. We ignore this warning.
    cmd.directory = locale_path  # pyright: ignore [reportAttributeAccessIssue]
    cmd.use_fuzzy = True
    cmd.finalize_options()
    try:
        cmd.run()
    except OptionError:
        pass


class CustomBuildHook(BuildHookInterface[BuilderConfigBound]):
    """Custom Hatch builld hook for localization using PyBabel."""

    @override
    def initialize(
        self,
        version: str,
        build_data: dict[str, object],
    ) -> None:
        """Initialize the build hook."""
        for locale_path in map(
            Path,
            (
                "src/whiteprints_template_context/locale",
                "whiteprints_template_context/locale",
            ),
        ):
            if locale_path.is_dir():
                _compile(locale_path)



# SPDX-FileCopyrightText: © 2024 The "Whiteprints template context" contributors <whiteprints@pm.me>
#
# SPDX-License-Identifier: MIT

"""Localization."""

import gettext
import pathlib
from typing import Final


__all__: Final = ["LOCALE_DIRECTORY", "TRANSLATION", "_"]
"""Public module attributes."""


LOCALE_DIRECTORY: Final = pathlib.Path(__file__).parent / "locale"
"""Path to the directory containing the locales."""

TRANSLATION: Final = gettext.translation(
    "messages",
    LOCALE_DIRECTORY,
    fallback=True,
)
"""A Gettext translation."""

_: Final = TRANSLATION.gettext
"""Convenient access to Gettext's translation."""



# SPDX-FileCopyrightText: © 2024 The "Whiteprints template context" contributors <whiteprints@pm.me>
#
# SPDX-License-Identifier: MIT

"""Top-level module."""

from typing import Final

from whiteprints_template_context.package_metadata import __version__


__all__: Final = ["__version__"]
"""Public module attributes."""



# SPDX-FileCopyrightText: © 2024 The "Whiteprints template context" contributors <whiteprints@pm.me>
#
# SPDX-License-Identifier: MIT
"""Update Copier context."""

from __future__ import annotations

import re
import sys
import unicodedata
from functools import lru_cache
from typing import Any, Final, cast

from copier_templates_extensions import ContextHook
from license_expression import (  # type: ignore [reportMissingTypeStubs]
    BaseSymbol,
    get_spdx_licensing,
)


if sys.version_info >= (3, 12):
    from typing import override
else:
    from typing_extensions import override


LATEST_PYTHON: Final = 13


def slugify(value: str, *, allow_unicode: bool = False) -> str:
    """Slugify.

    Convert to ASCII if "allow_unicode" is False. Convert spaces or repeated
    dashes to single dashes. Remove characters that aren"t alphanumerics,
    underscores, or hyphens. Convert to lowercase. Also strip leading and
    trailing whitespace, dashes, and underscores.

    Returns:
        a slug of `value`.
    """
    if allow_unicode:
        value = unicodedata.normalize("NFKC", value)
    else:
        value = (
            unicodedata.normalize("NFKD", value)
            .encode("ascii", "ignore")
            .decode("ascii")
        )
    value = re.sub(r"[^\w\s-]", "", value.lower())
    return re.sub(r"[-\s]+", "-", value).strip("-_")


# SPDX-SnippetEnd


@lru_cache
def spdx_symbols(expression: str) -> set[str]:
    """Extract SPDX symbols from a license expression.

    This function parses a given SPDX license expression and returns a set
    containing the symbols (licenses or exceptions) used in that expression.

    Arguments:
        expression: An SPDX license expression string, such as "MIT AND
           Apache-2.0".

    Returns:
        A set of SPDX symbols (e.g., {"MIT", "Apache-2.0"}) extracted from the
           input expression.
    """
    licensing = get_spdx_licensing()
    license_symbols = cast(
        list[BaseSymbol],
        licensing.license_symbols(  # type: ignore [reportGeneralTypeIssues]
            expression
        ),
    )
    return {symbol.obj for symbol in license_symbols}


class ContextUpdater(ContextHook):  # pylint: disable=abstract-method
    # Pylint tells that parse method need to be overriden, but is our case
    # it is not necessary.
    """Modify the Copier context with additional computed values.

    This method updates the context with various derived attributes based on
    the provided input context. It includes attributes like `latest_python`,
    `project_slug`, `package_name`, `target_python`, `tox_python_list`, and
    license-related information.

    Arguments:
        context: A dictionary representing the current context of the project,
            containing values like `project_name`, `target_python_version`,
            `code_license_id`, and `resources_license_id`.

    Returns:
        A dictionary containing the updated context values, including modified
        project settings and license information.
    """

    @staticmethod
    def _hook(context: dict[str, Any]) -> dict[str, Any]:
        latest_python = LATEST_PYTHON
        target_python_minor = context["target_python_version"][3:]

        context["latest_python"] = latest_python
        context["project_slug"] = slugify(context["project_name"])
        context["package_name"] = context["project_slug"].replace("-", "_")
        context["target_python"] = f"3.{target_python_minor}"
        context["tox_python_list"] = (
            "py{"
            + ",".join(
                f"3{python_minor}"
                for python_minor in range(
                    int(target_python_minor),
                    latest_python + 1,
                )
            )
            + "}"
        )
        code_license_symbols = list(spdx_symbols(context["code_license_id"]))
        context["code_license_symbols"] = list(code_license_symbols)
        context["resources_license_symbols"] = list(
            spdx_symbols(context["resources_license_id"])
        )
        context["code_license_text_ext"] = context["code_license_id"]
        for symbol in code_license_symbols:
            context["code_license_text_ext"] = context[
                "code_license_text_ext"
            ].replace(
                symbol, f"[{symbol}](https://spdx.org/licenses/{symbol})"
            )

        context["code_license_text"] = context["code_license_id"]
        for symbol in code_license_symbols:
            context["code_license_text"] = context[
                "code_license_text"
            ].replace(symbol, f"[{symbol}](../LICENSES/{symbol}.txt)")

        return context

    @override
    def hook(self, context: dict[str, Any]) -> dict[str, Any]:
        try:
            return self._hook(context)
        except KeyError:
            return {}
